// Libraries
import React, { Component } from 'react'
import PropTypes from 'prop-types'
import classNames from 'classnames'

export class AddressBook extends Component {
  constructor (props) {
    super(props)

    this.state = {
      loading: false
    }

    this.handleOnChange = this.handleOnChange.bind(this)
  }

  /**
   * Hide the loading element if the address has changed, or the form display
   * has changed
   * @param  {Object} prevProps
   */
  componentDidUpdate (prevProps) {
    if (prevProps.currentAddressId !== this.props.currentAddressId || prevProps.addressFormDisplayed !== this.props.addressFormDisplayed) {
      this.setState({
        loading: false
      })
    }
  }

  /**
   * Show the loading element then pass the addressId to the prop
   * @param  {Number} addressId
   */
  handleOnChange (addressId) {
    this.setState({
      loading: true
    })

    this.props.onBookAddressSelected(addressId)
  }

  /**
   * Renders the address company name if present
   * @param  {Object} address
   * @return {String} - HTML markup for the component
   */
  renderCompanyName (address) {
    const metaAttributes = address.meta_attributes
    if (metaAttributes && metaAttributes.company_name) {
      return <span>, &nbsp; { metaAttributes.company_name.value }</span>
    }
  }

  /**
   * Renders the address line 2 if present
   * @param  {Object} address
   * @return {String} - HTML markup for the component
   */
  renderAddressLine2 (address) {
    if ( address.address_line_2 ) {
      return <span>, &nbsp; { address.address_line_2 }</span>
    }
  }

  renderOptions (address) {
    const { currentAddressId, addressFormDisplayed, onAddressDeleted, deleteable } = this.props
    const isSelected = (!addressFormDisplayed && address.id === currentAddressId)
    const addressLabel = typeof address.meta_attributes.label === 'undefined' ? 'Other' : address.meta_attributes.label.value

    const optionLabel = <span><b>{ addressLabel }</b> -&nbsp; { address.first_name } { address.last_name } { this.renderCompanyName(address) },&nbsp;
      { address.address_line_1 } { this.renderAddressLine2(address) },&nbsp; { address.postcode },&nbsp; { address.city },&nbsp; { address.country }
    </span>

    return (
      <div className='c-address-book__radio-container'>
        <label
          htmlFor={address.id}
          className={classNames('c-address-book__radio-label', { 'c-address-book__radio-label--selected': isSelected })}
        >
          <input
            id={address.id}
            type='radio'
            name='address-book-radio'
            className='c-address-book__radio'
            onChange={() => this.handleOnChange(address.id)}
            checked={isSelected}
          />
          { optionLabel }
        </label>
        { deleteable && <a className='c-address-book__delete-address' label='Delete' onClick={() => onAddressDeleted(address)}>Delete</a> }
      </div>
    )
  }

  renderEntries () {
    // Sort array of objects and return a new array
    // with the preferred address as the first object.
    // We always want to display the preferred-address as the first option
    const entries = this.props.addressBook.sort(function (a, b) {
      return b.preferred_shipping - a.preferred_shipping
    })

    return (
      <div className='c-address-book__entries'>
        { entries.map(address => {
          return (
            <div key={address.id}>
              { this.renderOptions(address) }
            </div>
          )
        }) }
      </div>
    )
  }

  render () {
    const { onNewAddress, addressFormDisplayed } = this.props
    const { loading } = this.state

    return (
      <div className='c-address-book'>
        <h3>Your addresses</h3>
        { this.renderEntries() }
        { !addressFormDisplayed && <a className='c-address-book__new-address' onClick={onNewAddress}>+ Add a new address</a> }
        { loading && <div className='o-loading-overlay'><div className='o-loading' /></div> }
      </div>
    )
  }
}

AddressBook.propTypes = {
  addressBook: PropTypes.arrayOf(PropTypes.object),
  addressFormDisplayed: PropTypes.bool,
  currentAddressId: PropTypes.string,
  onAddressDeleted: PropTypes.func,
  onBookAddressSelected: PropTypes.func,
  onNewAddress: PropTypes.func,
  deleteable: PropTypes.bool
}
