// Libraries
import React, { Fragment, PureComponent } from 'react'
import classNames from 'classnames'
import PropTypes from 'prop-types'
import format from 'date-fns/format'
import t from 'typy'

// Libs
import businessDaysFromNow from '../../lib/business-days-from-now'
import { decimalPrice } from '../../lib/decimal-price'
import { Button } from '../../objects/button'
import { Flash } from '../../objects/flash'
import ShippingMethodsHeader from '../checkout/shipping-methods-header'

export class ShippingMethods extends PureComponent {
  /**
   * Renders the continue button
   */
  renderContinueButton () {
    const { buttonLabel, handleFormSubmit } = this.props
    return (
      <Button
        className='o-button--sml'
        aria-label={buttonLabel}
        label={buttonLabel}
        status='positive'
        type='submit' onClick={handleFormSubmit}
      />
    )
  }

  /**
   * Renders the shipping method form, which is a wrapper containing the
   * shipping methods
   * @return {string} - HTML markup for the component
   */
  renderForm () {
    return (
      <div className='c-shipping-method__list'>
        <form className='o-form__wrapper o-form__background'>
          <div>{ this.renderShippingMethods() }</div>
          <div className='c-shipping-method__continue-cta'>
            <div className='o-form__input-group'>
              { this.renderContinueButton() }
            </div>
          </div>
        </form>
      </div>
    )
  }

  /**
   * Renders the available shipping methods, which is a set of radio buttons
   * @return {string} - HTML markup for the component
   */
  renderShippingMethods () {
    const { cartShippingMethod, handleSetShippingMethod, shippingMethods } = this.props

    if (cartShippingMethod) {
      return shippingMethods.map((method) => {
        const deliveryDate = t(method, 'meta_attributes.working_days.value').safeString
        const deliveryDatePresent = t(method, 'meta_attributes.working_days.value').isDefined

        return (
          <div
            className='o-form__input-group c-shipping-method__list-input-group'
            key={method.id}
            onClick={() => handleSetShippingMethod(method)}
            aria-label={method.sku}
          >
            <label htmlFor={method.sku} className={classNames('c-shipping-method__radio', { 'c-shipping-method__radio--selected': (method.id === cartShippingMethod.id) })}>
              <input className='c-shipping-method__radio-input' id={`${method.sku}_${method.id}`} value={method.sku} type='radio'
                name='shipping_method'
                checked={method.id === cartShippingMethod.id} onChange={() => handleSetShippingMethod(method)} />
              <span className='c-shipping-method__radio-caption' />
              <span className='c-shipping-method__list-cost'>&pound;{ decimalPrice(method.total) }</span>
              <span className='c-shipping-method__list-title'>{ method.label }</span>
              { deliveryDatePresent ? this.renderDeliveryDate(deliveryDate) : null }
            </label>
          </div>
        )
      })
    }
  }

  renderDeliveryDate (deliveryDate) {
    const formattedDeliveryDate = format(businessDaysFromNow(deliveryDate), 'dddd Do MMMM')

    return (
      <Fragment>
        <span className='c-shipping-method__list-delivery-date-label'>Estimated Delivery: </span>
        <span className='c-shipping-method__list-delivery-date'>{ formattedDeliveryDate }</span>
      </Fragment>
    )
  }

  render () {
    const { errorMessage, shippingMethods } = this.props

    if (!shippingMethods) {
      return null
    }

    return (
      <div aria-label='Shipping Methods' className={classNames(this.props.className, 'o-form c-shipping-method')}>
        <ShippingMethodsHeader title={'Shipping Method'}/>
        { errorMessage && <Flash text={errorMessage} modifier={'error'}/> }
        { this.renderForm() }
      </div>
    )
  }
}

ShippingMethods.propTypes = {
  buttonLabel: PropTypes.string.isRequired,
  cartLineItemsCount: PropTypes.number,
  cartShippingMethod: PropTypes.object,
  errorMessage: PropTypes.string,
  handleFormSubmit: PropTypes.func,
  handleSetShippingMethod: PropTypes.func,
  shippingMethods: PropTypes.array
}
